#include <ansi_c.h>
#include <visa.h>
#include <cvirte.h>     
#include <userint.h>
#include "RSSFE_DVBT_Coder_Example.h"
#include "rssfe.h"

/* Error check macro for driver functions */

#define CHECKERR( fCal ) \
    { \
        error = (fCal);\
        rssfe_error_message (io, error, error_message);\
        if (error < VI_SUCCESS) {\
            sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", error, error_message);\
            MessagePopup ("Error", error_buffer); \
            rssfe_error_query (io, &error, error_message);\
            sprintf (error_buffer, "Secondary Error: 0x%08X, %s\n", error, error_message);\
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);\
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);\
            SetWaitCursor (0); \
            MessagePopup ("Error", error_buffer); \
            rssfe_close(io); \
            break; \
        } \
    }
    
/* Globals */

ViInt32 panelHandle;
ViSession io;
ViStatus    error;
ViChar      error_message[256];
ViChar      error_buffer[256];

/* Local Functions */
int initialize (void);
            
/*= Main Function ===========================================================*/   
int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    panelHandle = LoadPanel (0, "RSSFE_DVBT_Coder_Example.uir", PANEL); 
    DisplayPanel (panelHandle);  
    RunUserInterface ();
    DiscardPanel (panelHandle);
    return 0;
}


/*===========================================================================*/
/* Function: Exit                                                            */
/* Purpose:  This is a callback function of Exit button. It closes the       */
/*           session and exits the example.                                  */
/*===========================================================================*/
int CVICALLBACK exitApp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
        }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{

    if (panel == panelHandle)
        MessagePopup ("Help", "This is a simple demo program using\n"
                              "the Rohde&Schwarz SFU Broadcast Test System\n"
                              "instrument driver provided in the package.\n");

    return;
}
                     
               
/*===========================================================================*/
/* Function: Apply Settings                                                  */
/* Purpose:  This is a callback function of Apply button. The values are     */
/*           send to the instrument                                          */
/*===========================================================================*/
int CVICALLBACK apllySettings (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    
    ViReal64 freq, level;
    ViInt32 constellation, inSelectLP, modeLP, codeRateLP, sourceHP, sourceLP,
            inSelectHP, modeHP, codeRateHP, FFTMode,
            hierarchyVal = 0, constelVal = 0;
    
    ViBoolean modState, DVBHState,RFState, stuffingHP, stuffingLP;
  
    
    switch (event)
    {
        case EVENT_COMMIT:
             SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 1);
             SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 1);
             ProcessDrawEvents ();

             if (initialize() < VI_SUCCESS)
             {
                SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);
                break;
             }
             SetWaitCursor(1);
             
            
             GetCtrlVal (panelHandle,PANEL_FREQ         , &freq);
             GetCtrlVal (panelHandle,PANEL_LEVEL        , &level);
             GetCtrlVal (panelHandle,PANEL_CONSTELL     , &constellation);
             GetCtrlVal (panelHandle,PANEL_IN_SELECT_LP , &inSelectLP);
             GetCtrlVal (panelHandle,PANEL_CODE_RATE_LP , &codeRateLP);
             GetCtrlVal (panelHandle,PANEL_MODE_LP      , &modeLP);
             GetCtrlVal (panelHandle,PANEL_IN_SELECT_HP , &inSelectHP);
             GetCtrlVal (panelHandle,PANEL_CODE_RATE_HP , &codeRateHP);
             GetCtrlVal (panelHandle,PANEL_MODE_HP      , &modeHP);    
             GetCtrlVal (panelHandle,PANEL_FFT_MODE     , &FFTMode);
             GetCtrlVal (panelHandle,PANEL_MOD_STATE    , &modState);
             GetCtrlVal (panelHandle,PANEL_DVBH_STATE   , &DVBHState);
             GetCtrlVal (panelHandle,PANEL_RF_STATE     , &RFState);
             GetCtrlVal (panelHandle,PANEL_STUFFING_HP  , &stuffingHP);
             GetCtrlVal (panelHandle,PANEL_STUFFING_LP  , &stuffingLP);
             GetCtrlVal (panelHandle,PANEL_SOURCE_HP    , &sourceHP);
             GetCtrlVal (panelHandle,PANEL_SOURCE_LP    , &sourceLP);
             
             CHECKERR (rssfe_confTxFreq (io, freq));
             CHECKERR (rssfe_confTxLevel (io, level, RFState, 20.0,
                                         RSSFE_RFOUT_AMODE_AUTO, RSSFE_VAL_POW_LEV_DBM));
             CHECKERR (rssfe_confDVBTInputSignal (io, RSSFE_VAL_PRIORITY_HIGH, sourceHP,
                                                inSelectHP, stuffingHP));
   
             switch (constellation){
                case  0:
                case  1: 
                case  2: 
                        hierarchyVal = 0; 
                        constelVal = constellation;
                break;
        
                case  3:
                case  4: 
                case  5: 
                        hierarchyVal = constellation - 2; 
                        constelVal = 1;
                break;
        
                case  6:
                case  7: 
                case  8: 
                        hierarchyVal = constellation - 5; 
                        constelVal = 2;
                break;
             }    
             
             CHECKERR (rssfe_confDVBTCoding (io, RSSFE_IQC_DVBT_CHBW6,
                                            FFTMode, RSSFE_IQC_DVBT_GUAR_14,
                                            hierarchyVal, constelVal));
    
             CHECKERR (rssfe_confDVBTCodeRate (io, RSSFE_VAL_PRIORITY_HIGH, codeRateHP));
             CHECKERR (rssfe_confDVBTTestSignal (io, RSSFE_VAL_PRIORITY_HIGH, modeHP));
    
             if (constellation > 2) {
                CHECKERR (rssfe_confDVBTInputSignal (io, RSSFE_VAL_PRIORITY_LOW, sourceLP,
                                                inSelectLP, stuffingLP));
                CHECKERR (rssfe_confDVBTCodeRate (io, RSSFE_VAL_PRIORITY_LOW, codeRateLP));
                CHECKERR (rssfe_confDVBTTestSignal (io, RSSFE_VAL_PRIORITY_LOW, modeLP));
             }
    
             CHECKERR (rssfe_confTxModulation (io, modState, RSSFE_VAL_DM_SOURCE_DTV,
                                               RSSFE_VAL_DM_STANDARD_DVBT,
                                               RSSFE_VAL_DM_SPEC_POLAR_NORM));
             CHECKERR (rssfe_confDVBHCoding (io, DVBHState, RSSFE_IQC_DVBT_SYMB_NAT));
 
             rssfe_close(io);
             
             SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
             SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);
             SetWaitCursor(0);
            
            break;
    }
    return 0;
}


/*===========================================================================*/ 
/* Local Functions Definitions                                               */
/*===========================================================================*/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function initializes the instrument, performs ID Query     */
/*           and Reset                                                       */
/*===========================================================================*/
int initialize (void)
{
    ViChar      instr_desc[256];
    ViBoolean   id;
    ViBoolean   rst;
    

    SetWaitCursor(0);
    GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
    GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
    GetCtrlVal (panelHandle, PANEL_RESET, &rst);
    SetWaitCursor(1);
    if ((error = rssfe_init (instr_desc, id, rst, &io)) < 0 )
    {
        rssfe_error_message(VI_NULL, error, error_message);
        sprintf (error_buffer, "Initialization Error:\n\n%s\n\nCheck your "
                "connections and make sure you have the right instrument descriptor.",
                 error_message);
        MessagePopup ("ERROR!", error_buffer);
        SetWaitCursor(0);
        return(-1);   
    }

    viSetAttribute (io, VI_ATTR_TMO_VALUE, 10000);
    SetWaitCursor(0);
    return 0;
}

